import os

import pyglet
from pyglet import resource
from pyglet import graphics
from pyglet import text
from pyglet import sprite
from pyglet import gl

from bruce import config
from bruce import page

# XXX this doesn't appear to be working... I still see an artifact on the very
# edge of a blown-up image
class ClampTextureImageGroup(graphics.Group):
    def set_state(self):
        gl.glPushAttrib(gl.GL_TEXTURE_BIT)
        gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_WRAP_S,
            gl.GL_CLAMP_TO_EDGE)
        gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_WRAP_T,
            gl.GL_CLAMP_TO_EDGE)

    def unset_state(self):
        gl.glPopAttrib()

class ImagePage(page.Page):
    flags = (
        ('zoom', bool, False),
        ('title', str, ''),
        ('caption', str, ''),
    )
    name = 'image'

    @classmethod
    def as_html(cls, content, **flags):
        path = os.path.join(config.get('directory'),
            page.decode_content(cls, content))
        l = []
        if 'title' in flags:
            l.append('<b>%s</b>'%flags['title'])
        l.append('<img height="100" src="%s">'%path)
        if 'caption' in flags:
            l.append(flags['caption'])
        return '<br>'.join(l)

    title_label = caption_label = None
    def on_enter(self, vw, vh):
        self.batch = graphics.Batch()

        # load the image
        path = self.content.strip()
        if os.path.exists(path):
            im = pyglet.image.load(path)
        else:
            im = resource.image(path)
        self.image_width = im.width
        self.image_height = im.height
        im.anchor_x = im.width//2
        im.anchor_y = im.height//2
        self.image = sprite.Sprite(im, 0, 0, group=ClampTextureImageGroup(),
            batch=self.batch)

        # if we have a title then generate it
        if self.title:
            self.title_label = text.Label(self.title,
                font_name=self.cfg['title.font_name'],
                font_size=self.cfg['title.font_size'],
                color=self.cfg['title.color'],
                halign='center', valign='bottom', batch=self.batch)

        # if we have a captio then generate it
        if self.caption:
            self.caption_label = text.Label(self.caption,
                font_name=self.cfg['text.font_name'],
                font_size=self.cfg['text.font_size'],
                color=self.cfg['text.color'],
                halign='center', valign='bottom', batch=self.batch)

        # position all the elements
        self.on_resize(vw, vh)

    def on_resize(self, vw, vh):
        # position the labels and adjust the available viewport height
        if self.title_label:
            self.title_label.x = vw//2
            self.title_label.y = vh = vh - self.title_label.content_height
        yoffset = 0
        if self.caption_label:
            self.caption_label.x = vw//2
            self.caption_label.y = 0
            vh -= self.caption_label.content_height
            # shift up to avoid the caption
            yoffset += self.caption_label.content_height

        # make width and height even to avoid placing the image on a
        # half-pixel bounday when centered
        vw -= vw %2
        vh -= vh %2

        # determine whether we're scaling
        w, h = self.image_width, self.image_height
        if self.zoom or w > vw or h > vh:
            self.image.scale = min(vw / float(w), vh / float(h))

        # and center the image
        self.image.position = (vw//2, yoffset + vh//2)

    def on_leave(self):
        self.image = None
        self.batch = None
        self.title_label = None
        self.caption_label = None

    def draw(self):
        self.batch.draw()

config.add_section('image', {
    'text.font_name': 'Arial',
    'text.font_size': 64,
    'text.color': (255, 255, 255, 255),
    'title.font_name': 'Arial',
    'title.font_size': 64,
    'title.color': (200, 255, 255, 255),
})
